@@ -1,3 +1,20 @@
+# GNOME Specimen, a font preview application for GNOME
+# Copyright (C) 2006  Wouter Bolsterlee <uws@xs4all.nl>
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License
+# as published by the Free Software Foundation; either version 2
+# of the License, or (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+
 
 import os.path
 
@@ -6,17 +23,26 @@ import gtk
 import gtk.gdk
 import gtk.glade
 import pango
+import gconf
 
 from gettext import gettext as _
 
-import config
+import specimen.config as config
 
 class SpecimenWindow:
 
     families = []
 
+    # Note to translators: this should be a pangram (a sentence containing all
+    # letters of your alphabet. See http://en.wikipedia.org/wiki/Pangram for
+    # more information and possible samples for your language.
+    preview_text = _('The quick brown fox jumps over the lazy dog.')
     preview_size = 12
-    preview_text = _('Pack my box with five dozen liquor jugs.')
+    # IMPORTANT: Keep the above two settings in sync with the GConf schema
+    # file. The translated string is used in case no user changes have been
+    # made to the GConf keys.
+
+    # default colors
     preview_fgcolor = gtk.gdk.color_parse('black')
     preview_bgcolor = gtk.gdk.color_parse('white')
 
@@ -53,6 +79,11 @@ class SpecimenWindow:
         'smallcaps',
     ]
 
+    # GConf paths
+    gconf_path_namespace = '/apps/gnome-specimen'
+    gconf_path_preview_text = '/apps/gnome-specimen/preview_text'
+    gconf_path_preview_size = '/apps/gnome-specimen/preview_size'
+
     def __init__(self):
         'Initializes the application'
 
@@ -74,18 +105,29 @@ class SpecimenWindow:
             'remove': tree.get_widget('remove-button'),
             'clear': tree.get_widget('clear-button'),
         }
-        self.update_button_sensitivity()
+        self.update_ui_sensitivity()
 
-        # update
-        self.on_preview_size_changed(self.preview_size_spinbutton)
-        self.on_preview_text_changed(self.preview_text_entry)
-        self.schedule_update_previews()
+        # gconf
+        self.gconf_client = gconf.client_get_default()
+        self.gconf_client.add_dir(self.gconf_path_namespace, gconf.CLIENT_PRELOAD_NONE)
+        self.gconf_client.notify_add(self.gconf_path_namespace, self.on_gconf_key_changed)
+        self.gconf_client.notify(self.gconf_path_preview_text)
+        self.gconf_client.notify(self.gconf_path_preview_size)
 
         # show the window
         self.window.show_all()
 
     def quit(self):
         'Quits the application'
+
+        # Store current values in GConf
+        self.gconf_client.set_int(self.gconf_path_preview_size, self.preview_size)
+        if self.preview_text.strip() == '': # reset to default:
+            self.gconf_client.unset(self.gconf_path_preview_text)
+        else:
+            self.gconf_client.set_string(self.gconf_path_preview_text, self.preview_text)
+
+        # Quit the application
         gtk.main_quit()
 
     def on_destroy_event(self, widget, data=None):
@@ -125,7 +167,7 @@ class SpecimenWindow:
         # setup the treeselection
         self.fonts_treeview_selection = self.fonts_treeview.get_selection()
         self.fonts_treeview_selection.set_mode(gtk.SELECTION_SINGLE)
-        self.fonts_treeview_selection.connect('changed', self.update_button_sensitivity)
+        self.fonts_treeview_selection.connect('changed', self.update_ui_sensitivity)
 
         # setup interaction
         self.fonts_treeview.connect('row-activated', self.on_row_activated)
@@ -238,6 +280,10 @@ class SpecimenWindow:
         # setup the treeselection
         self.previews_treeview_selection = self.previews_treeview.get_selection()
         self.previews_treeview_selection.set_select_function(self._set_preview_row_selection)
+        self.previews_treeview_selection.connect('changed', self.update_ui_sensitivity)
+
+        self.previews_treeview.connect('scroll-event', self.on_previews_treeview_scroll_event)
+
 
     def cell_data_cb(self, column, cell, model, treeiter, data=None):
         if model.get_path(treeiter)[0] % 2 == 0:
@@ -270,10 +316,10 @@ class SpecimenWindow:
     def _set_cell_attributes_for_name_cell(self, cell, name):
         try:
             # set the values
+            font_desc, size, ellipsize = self.name_cell_properties
             cell.set_property('text', name)
-            background, foreground, font_desc, size, ellipsize = self.name_cell_properties
-            cell.set_property('background', background)
-            cell.set_property('foreground', foreground)
+            cell.set_property('background', None)
+            cell.set_property('foreground', None)
             cell.set_property('font-desc', font_desc)
             cell.set_property('size', size)
             cell.set_property('ellipsize', ellipsize)
@@ -283,8 +329,6 @@ class SpecimenWindow:
             font_desc = self.window.get_pango_context().get_font_description()
             size = font_desc.get_size()
             self.name_cell_properties = (
-                    'grey',
-                    'black',
                     font_desc,
                     size,
                     cell.get_property('ellipsize')
@@ -299,7 +343,24 @@ class SpecimenWindow:
         cell.set_property('foreground-gdk', self.preview_fgcolor)
         cell.set_property('font-desc', font_description)
         cell.set_property('size', self.preview_size * pango.SCALE)
-        cell.set_property('ellipsize', pango.ELLIPSIZE_END)
+        cell.set_property('ellipsize', pango.ELLIPSIZE_NONE)
+
+    def add_preview_from_path(self, path):
+        model = self.fonts_treeview.get_model()
+        if len(path) == 1:
+            # add all child rows
+            treeiter = model.iter_children(model.get_iter(path))
+            while treeiter is not None:
+                family, face = model.get(treeiter, 1, 2)
+                self.add_preview(family, face)
+                treeiter = model.iter_next(treeiter)
+
+        else:
+            # this is a child row
+            treeiter = model.get_iter(path)
+            family, face = model.get(treeiter, 1, 2)
+            self.add_preview(family, face)
+
 
     def add_preview(self, family, face):
         'Adds a preview to the list of previews'
@@ -316,7 +377,7 @@ class SpecimenWindow:
         # Select the new entry in the preview listing, so that it can be quickly removed using delete
         self.select_last_preview()
 
-        self.update_button_sensitivity()
+        self.update_ui_sensitivity()
 
     def schedule_update_previews(self, *args):
         'Schedules an update of the previews'
@@ -331,7 +392,7 @@ class SpecimenWindow:
         self.previews_preview_column.queue_resize()
         self.previews_treeview.queue_draw()
 
-        self.update_button_sensitivity()
+        self.update_ui_sensitivity()
 
         # Allow this method to be used as a single-run idle timeout
         return False
@@ -339,7 +400,7 @@ class SpecimenWindow:
     def clear_previews(self):
         'Clears all previews'
         self.previews_store.clear()
-        self.update_button_sensitivity()
+        self.update_ui_sensitivity()
 
     def num_previews(self):
         'Returns the number of previews'
@@ -376,23 +437,10 @@ class SpecimenWindow:
                 # new bottom font name row.
                 self.select_last_preview()
 
-        self.update_button_sensitivity()
+        self.update_ui_sensitivity()
 
     def on_row_activated(self, treeview, path, viewcolumn, *user_data):
-        if len(path) == 1:
-            # this is a parent row, expand/collapse
-            is_expanded = treeview.row_expanded(path)
-            if is_expanded:
-                treeview.collapse_row(path)
-            else:
-                treeview.expand_row(path, False)
-
-        else:
-            # this is a child row
-            model = treeview.get_model()
-            treeiter = model.get_iter(path)
-            family, face = model.get(treeiter, 1, 2)
-            self.add_preview(family, face)
+        self.add_preview_from_path(path)
 
     def on_preview_size_changed(self, widget, user_data=None):
         'Callback for changed preview point size'
@@ -431,6 +479,22 @@ class SpecimenWindow:
         # propagate the event
         return False
 
+    def on_previews_treeview_scroll_event(self, treeview, event, data=None):
+        '''Change the preview size when a mouse wheel event (with Control key)
+        was received for the previews treeview.'''
+
+        # Only act if the Control key is pressed
+        if event.state & gtk.gdk.CONTROL_MASK:
+            if event.direction == gtk.gdk.SCROLL_UP:
+                self.preview_size_spinbutton.set_value(self.preview_size + 1)
+            elif event.direction == gtk.gdk.SCROLL_DOWN:
+                self.preview_size_spinbutton.set_value(self.preview_size - 1)
+
+            # We handled this event
+            return True
+
+        # Propagate further in all other cases
+        return False
 
     # preview colors
 
@@ -451,7 +515,7 @@ class SpecimenWindow:
         except (AttributeError):
             # Create the dialog
             self.colors_dialog = gtk.Dialog(
-                    _('Change colors...'),
+                    _('Change colors'),
                     self.window,
                     gtk.DIALOG_DESTROY_WITH_PARENT,
                     (gtk.STOCK_CLOSE, gtk.RESPONSE_CANCEL))
@@ -507,15 +571,34 @@ class SpecimenWindow:
         self.set_colors(fgcolor, bgcolor)
 
 
+    # gconf
+
+    def on_gconf_key_changed(self, client, connection_id, entry, user_data=None):
+        key_name = entry.get_key()
+        if key_name == self.gconf_path_preview_text:
+            # Set the text from gconf, but make sure translations work correctly
+            text = self.gconf_client.get_string(self.gconf_path_preview_text)
+            if text is not None and _(text) != self.preview_text:
+                # There is a value in GConf that differs from the default; use it
+                self.preview_text = text
+            self.preview_text_entry.set_text(self.preview_text)
+        elif key_name == self.gconf_path_preview_size:
+            # Don't bother going the hard way for the size :)
+            size = self.gconf_client.get_int(self.gconf_path_preview_size)
+            if size > 0:
+                self.preview_size = size
+            self.preview_size_spinbutton.set_value(self.preview_size)
+
+        self.update_previews()
+
     # buttons
 
     def on_add_button_clicked(self, widget, data=None):
         'Callback for the Add button'
         model, treeiter = self.fonts_treeview.get_selection().get_selected()
-        if treeiter is not None:
-            family, face = model.get(treeiter, 1, 2)
-            self.add_preview(family, face)
-            self.fonts_treeview.grab_focus()
+        path = model.get_path(treeiter)
+        self.add_preview_from_path(path)
+        self.fonts_treeview.grab_focus()
 
     def on_remove_button_clicked(self, widget, data=None):
         'Callback for the Remove button'
@@ -528,21 +611,23 @@ class SpecimenWindow:
         self.clear_previews()
         self.fonts_treeview.grab_focus()
 
-    def update_button_sensitivity(self, *args):
-        'Updates the button sensitivity'
+    def update_ui_sensitivity(self, *args):
+        'Updates the user interface sensitivity'
 
-        # The Add button should only be sensitive if a valid font style is
-        # selected font listing in the left pane. This is detected by checking
-        # whether the current selection, if any, has path length 2 (ie. only
-        # child rows, not top level rows)
+        # The Add button is only sensitive if a font is selected in the fonts
+        # pane (left pane)
         model, rows = self.fonts_treeview.get_selection().get_selected_rows()
-        add_enabled = (len(rows) and len(rows[0]) == 2)
+        add_enabled = (len(rows) > 0)
         self.buttons['add'].set_sensitive(add_enabled)
 
-        # The Remove and Clear buttons should only be sensitive if the list of
-        # previews is not empty.
+        # The Remove button is only sensitive if a font is selected in the
+        # preview pane (right pane)
+        model, rows = self.previews_treeview.get_selection().get_selected_rows()
+        remove_enabled = (len(rows) > 0)
+        self.buttons['remove'].set_sensitive(remove_enabled)
+
+        # The Clear button is only sensitive if the number of previews > 0
         has_previews = (self.num_previews() > 0)
-        self.buttons['remove'].set_sensitive(has_previews)
         self.buttons['clear'].set_sensitive(has_previews)
 
 
@@ -585,13 +670,15 @@ class SpecimenWindow:
             self.about_dialog
 
         except (AttributeError):
-            name = _('GNOME Specimen')
+            name = 'GNOME Specimen'
             version = config.VERSION
             comments = _('Preview and compare fonts')
             copyright = u'Copyright \u00A9 2006 Wouter Bolsterlee'
             authors = ['Wouter Bolsterlee (wbolster@gnome.org)']
+            pixmap = gtk.gdk.pixbuf_new_from_file(os.path.join(config.PKGDATADIR, 'gnome-specimen-about.png'))
+            # Note to translators: translate this into your full name. It will
+            # be displayed in the application's about dialog.
             translators = _('translator-credits')
-            pixmap = gtk.gdk.pixbuf_new_from_file(os.path.join(config.PKGDATADIR, 'gnome-specimen.png'))
 
             self.about_dialog = gtk.AboutDialog()
             self.about_dialog.set_transient_for(self.window)
@@ -600,8 +687,8 @@ class SpecimenWindow:
             self.about_dialog.set_comments(comments)
             self.about_dialog.set_copyright(copyright)
             self.about_dialog.set_authors(authors)
-            self.about_dialog.set_translator_credits(translators)
             self.about_dialog.set_logo(pixmap)
+            self.about_dialog.set_translator_credits(translators)
 
             # just hide the about_dialog after first usage
             self.about_dialog.connect('response', lambda widget, response: widget.hide())
@@ -611,4 +698,3 @@ class SpecimenWindow:
 
         self.about_dialog.show()
         self.about_dialog.present()
-
